package interfacesFerrovia;

/**
 * Interface a ser implementada pelo Centro de Controle, que ser o responsvel pelo tratamento
 * de eventos gerados pela malha ferroviria.
 *   Define os mtodos chamados pela malha ferroviria na ocorrncia de cada evento gerado pela malha..
 *   Todo evento tem as seguintes informaes:
 *   - tipo do evento: o tipo de evento  implcito pelo mtodo sendo chamado.
 *   - id_origem: identificao nica do elemento que gerou o evento (composio ou trecho de ferrovia)
 *   - instante da ocorrncia do evento
 *   - dados do evento (cada tipo de evento tem um conjunto prprio de dados, descritos junto com cada 
 *     mtodo.
 * @author Fernando Vanini
 *
 */

public interface TratadorDeEventos extends DefsGlobais{
	
	/**
	 * Evento de alarme gerado por uma composio, por falha de operao
	 * @param idComposicao   Identificao nica da composio
	 * @param instanteAtual  Instante da ocorrncia do evento
	 * @param tipoDeFalha    Tipo da falha: (Acelerao, Freio, Outra [ ver definies em DefsGlobais.java])
	 */
	public void alarmeComposicao(String idComposicao, int instanteAtual, TipoFalha tipoDeFalha );
	
	/**
	 * Evento gerado pela composio quando a mesma inicia sua viagem.
	 * @param idComposicao   Identificao nica da composio
	 * @param instanteAtual  Instante da ocorrncia do evento
	 * @param idTrecho       Identificao nica do trecho de onde a composio parte.
	 */
	public void partidaComposicao(String idComposicao, int instanteAtual, String idTrecho);
	
	/**
	 * Evento gerado pela composio quando esta chega ao seu destino.
	 * @param idComposicao  Identificao nica da composio.
	 * @param instanteAtual Instante da ocorrncia do evento.
	 * @param idTrecho      Identificao do trecho pelo qual ocorreu a chegada.
	 */
	public void chegadaComposicao(String idComposicao, int instanteAtual, String idTrecho);
	
	/**
	 * Evento de alarme gerado pela chave, por falha de operao.
	 * @param idChave         Identificao nica da chave que gerou o alarme.
	 * @param instanteAtual   Instante da ocorrncia do evento.
	 * @param tipoDeFalha     Tipo de falha da chave.
	 */
	public void alarmeChave(String idChave, int instanteAtual, TipoFalha tipoDeFalha );	
	
	/**
	 * Evento gerado pela passagem de uma composio por um sensor.
	 * @param idTrecho       Identificao nica do trecho que gerou o evento.
	 * @param instanteAtual  Instante da ocorrncia do evento.
	 * @param posicao        Posio do sensor que gerou o evento, relativa  extremidade A do sensor.
	 * @param idComposicao   Identificao nica da composio.
	 * @param velocidade     Velocidade da composio no instante da passagem.
	 * @param sentidoAB      Sentido da passagem (AB => true, BA => false)
	 */
	public void passagemComposicao(String idTrecho, int instanteAtual, float posicao, String idComposicao, float velocidade, boolean sentidoAB);
	
	/**
	 * Evento gerado pela entrada de uma composio numa chave.
	 * @param idChave           Identificao nica da chave que gerou o evento.
	 * @param instanteAtual     Instante da ocorrncia do evento.
	 * @param idComposicao      Identificao da composio que entrou na chave
	 * @param idTrechoOrigem    Identificao do trecho de onde veio a composio
	 */
	public void entradaComposicaoChave(String idChave, int instanteAtual, String idComposicao, String idTrechoOrigem);

	/**
	 * Evento gerado pela sada de uma composio de uma chave.
	 * @param idChave          Identificao nica da chave que gerou o evento.
	 * @param instanteAtual     Instante da ocorrncia do evento.
	 * @param idComposicao      Identificao da composio que saiu da chave
	 * @param idTrechoDestino   Identificao do trecho para onde est indo a composio
	 */
	public void saidaComposicaoChave(String idChave, int instanteAtual, String idComposicao, String idTrechoDestino);
	
	/**
	 * Evento gerado a cada 'tic' do relgio do simulador.
	 * @param instanteAtual instante em que o 'tic' ocorre (relativo ao incio da simulao)
	 */
	public void tic(int instanteAtual);
	
}
